@@ -1,6 +1,6 @@
 /*
  *  Declarations for unit-test utilities for cryptmoumt
- *  (C)Copyright 2006-2011, RW Penney
+ *  (C)Copyright 2006-2012, RW Penney
  */
 
 /*
@@ -73,6 +73,16 @@ extern cm_testinfo_t *test_ctxtptr;
         return CM_TEST_FAILED; \
     }
 
+#define CM_ASSERT_STR_EQUAL(expected, actual) \
+    if (strcmp(expected, actual) != 0) { \
+        fprintf(stderr, "  test failed %s:%d  (\"%s\" != \"%s\")\n", \
+                 __FILE__, __LINE__, expected, actual); \
+        if (test_ctxtptr->current != NULL) { \
+            fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
+        ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
+        return CM_TEST_FAILED; \
+    }
+
 #define CM_ASSERT_DIFFERENT(expected, actual) \
     if ((expected) == (actual)) { \
         fprintf(stderr, "  test failed %s:%d\n  (%s == %s)", \
@@ -115,5 +125,5 @@ int cm_run_tests();
 #endif  /* _CMTEST_H */
 
 /*
- *  (C)Copyright 2006-2011, RW Penney
+ *  (C)Copyright 2006-2012, RW Penney
  */
