@@ -1,6 +1,6 @@
 /*
  *  Config-table and mount-table utilities for cryptmount
- *  (C)Copyright 2005-2011, RW Penney
+ *  (C)Copyright 2005-2012, RW Penney
  */
 
 /*
@@ -24,6 +24,8 @@
 #include <config.h>
 
 #include <ctype.h>
+#include <grp.h>
+#include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -56,9 +58,6 @@ struct statfile     /* information about status file */
 };
 
 
-static tgtdefn_t *parse_stream(FILE *fp, const char *cfgname);
-
-
 /*  The format for the status-file (version 0) is as follows:
     # [TITLE]           - one-line text header
     0                   - integer file-format version number
@@ -67,8 +66,84 @@ static tgtdefn_t *parse_stream(FILE *fp, const char *cfgname);
 */
 
 
+struct vardefn
+{
+    char *varname;
+    char *value;
+    struct vardefn *next;
+};
+static struct vardefn *vardefns = NULL;
+
+
+static tgtdefn_t *parse_stream(FILE *fp, const char *cfgname);
+
+
+void set_variable(char *varname, char *value)
+{   struct vardefn *newvar;
+
+    newvar = (struct vardefn*)malloc(sizeof(struct vardefn));
+    newvar->next = vardefns;
+
+    newvar->varname = cm_strdup(varname);
+    newvar->value = cm_strdup(value);
+
+    vardefns = newvar;
+}
+
+
+const char *query_variable(char *varname)
+{   struct vardefn *var;
+
+    for (var=vardefns; var!=NULL; var=var->next) {
+        if (strcmp(varname, var->varname) == 0) return var->value;
+    }
+
+    return NULL;
+}
+
+
+void init_env_dictionary()
+    /** Initialize internal dictionary of quasi-environmental variables */
+{   char buff[512];
+    const uid_t uid = getuid();
+    const gid_t gid = getgid();
+    struct passwd *pwent = NULL;
+    struct group *grent = NULL;
+
+    if (vardefns != NULL) return;
+
+    sprintf(buff, "%d", (int)uid);
+    set_variable("UID", buff);
+
+    pwent = getpwuid(uid);
+    if (pwent != NULL) {
+        set_variable("USERNAME", pwent->pw_name);
+        set_variable("HOME", pwent->pw_dir);
+    }
+
+    sprintf(buff, "%d", (int)gid);
+    set_variable("GID", buff);
+
+    grent = getgrgid(gid);
+    if (grent != NULL) set_variable("GROUPNAME", grent->gr_name);
+}
+
+
+void clear_env_dictionary()
+{   struct vardefn *vd;
+
+    while (vardefns != NULL) {
+        vd = vardefns;
+        if (vd->varname != NULL) free((void*)vd->varname);
+        if (vd->value != NULL) free((void*)vd->value);
+        vardefns = vd->next;
+        free((void*)vd);
+    }
+}
+
+
 tgtdefn_t *alloc_tgtdefn(const tgtdefn_t *prototype)
-    /* Allocate storage for a target-structure */
+    /** Allocate storage for a target-structure */
 {   tgtdefn_t *tgt;
 
 #define DFLT_OR_PROTO(dflt, proto) \
@@ -107,7 +182,7 @@ tgtdefn_t *alloc_tgtdefn(const tgtdefn_t *prototype)
 
 
 const tgtdefn_t *get_tgtdefn(const tgtdefn_t *head, const char *ident)
-    /* Find info-structure for target of given name */
+    /** Find info-structure for target of given name */
 {   const tgtdefn_t *itr,*ent=NULL;
 
     for (itr=head; itr!=NULL && ent==NULL; itr=itr->nx) {
@@ -121,7 +196,7 @@ const tgtdefn_t *get_tgtdefn(const tgtdefn_t *head, const char *ident)
 
 
 tgtdefn_t *clone_tgtdefn(const tgtdefn_t *orig)
-    /* Create (deep) copy of target-definition */
+    /** Create (deep) copy of target-definition */
 {   tgtdefn_t *clone;
 
     if (orig == NULL) return NULL;
@@ -157,7 +232,7 @@ tgtdefn_t *clone_tgtdefn(const tgtdefn_t *orig)
 
 
 void free_tgtdefn(tgtdefn_t *tgt)
-    /* Relinquish storage for a target-structure */
+    /** Relinquish storage for a target-structure */
 {
     if (tgt->ident != NULL) free((void*)tgt->ident);
 
@@ -180,7 +255,7 @@ void free_tgtdefn(tgtdefn_t *tgt)
 
 
 static void append(char c, char **buff, unsigned *pos, unsigned *bufflen)
-    /* Append character to string, reallocating memory as necessary */
+    /** Append character to string, reallocating memory as necessary */
 {   unsigned newlen;
 
     if (*pos >= *bufflen) {
@@ -194,12 +269,94 @@ static void append(char c, char **buff, unsigned *pos, unsigned *bufflen)
 }
 
 
-static int proc_string(void *ptr, const char *src)
-    /* Process var=val entry in config-table for "string" type */
-{   char **addr=(char**)ptr;
+void expand_variables(char **buff, const char *src)
+    /** Expand $(VAR) patterns within a string */
+{   const size_t srclen = strlen(src);
+    enum { S_PLAIN, S_VAR };
+    const char *curs = src;
+    char *varname, *varp;
+    int literal = 0;
+    unsigned state;
+    cm_string_t *result;
 
-    *addr = (char*)realloc(*addr, (size_t)(strlen(src) + 1));
-    strcpy(*addr, src);
+    result = cm_str_alloc(64);
+
+    state = S_PLAIN;
+    varname = (char*)malloc(srclen * 2);
+    for (curs=src; *curs!='\0'; ++curs) {
+        if (!literal && *curs == '\\') {
+            literal = 1;
+            continue;
+        }
+
+        switch (state) {
+            case S_PLAIN:
+                if (!literal && curs[0] == '$' && curs[1] == '(') {
+                    /* Mark start of variable term */
+                    state = S_VAR;
+                    varp = varname;
+                    ++curs;
+                } else {
+                    /* Copy character unchanged into output */
+                    cm_str_append_char(result, *curs);
+                }
+                break;
+            case S_VAR:
+                if (!literal && *curs == ')') {
+                    /* Extract value from dictionary */
+                    *varp = '\0';
+                    cm_str_append_str(result, query_variable(varname));
+                    state = S_PLAIN;
+                } else {
+                    /* accumulate characters of variable name */
+                    *varp = *curs;
+                    ++varp;
+                }
+                break;
+            default:
+                break;
+        }
+
+        literal = 0;
+    }
+    free((void*)varname);
+
+    if (*buff != NULL) free((void*)(*buff));
+    *buff = cm_str_strip(result);
+}
+
+
+#ifdef TESTING
+
+int tb_test_expand()
+    /** Check variable expansion in configuration-file parser */
+{   char answer[1024], *buff = NULL;
+
+    CM_TEST_START("Variable expansion");
+
+    expand_variables(&buff, "nothing here");
+    CM_ASSERT_STR_EQUAL("nothing here", buff);
+
+    expand_variables(&buff, "nothing\\ here");
+    CM_ASSERT_STR_EQUAL("nothing here", buff);
+
+    expand_variables(&buff, "nothing-$(UID) here");
+    sprintf(answer, "nothing-%d here", getuid());
+    CM_ASSERT_STR_EQUAL(answer, buff);
+
+    if (buff != NULL) free((void*)buff);
+
+    CM_TEST_OK();
+}
+
+#endif  /* TESTING */
+
+
+static int proc_string(void *ptr, const char *src)
+    /** Process var=val entry in config-table for "string" type */
+{   char **addr = (char**)ptr;
+
+    expand_variables(addr, src);
 
     return (*addr == NULL);
 }
@@ -207,7 +364,7 @@ static int proc_string(void *ptr, const char *src)
 
 #define PROC_VARVAL(FN_NAME, TYPE, FMT)                         \
     static int FN_NAME(void *ptr, const char *src)              \
-        /* Process var=val entry in config-table for "TYPE" */  \
+        /** Process var=val entry in config-table for "TYPE" */ \
     {   TYPE qv, *addr=(TYPE*)ptr;                              \
         if (sscanf(src, FMT, &qv) == 1) {                       \
             *addr = qv; return 0;                               \
@@ -223,7 +380,7 @@ PROC_VARVAL(proc_int64, int64_t, "%" SCNi64)
 
 
 int proc_flags(void *ptr, const char *src)
-    /* Convert string of configuration-switches into binary flags */
+    /** Convert string of configuration-switches into binary flags */
 {   unsigned *flags=(unsigned*)ptr;
     struct opt_t {
         const char *str;
@@ -264,7 +421,7 @@ int proc_flags(void *ptr, const char *src)
 
 
 static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
-    /* Process token (word) from configuration-file while parsing */
+    /** Process token (word) from configuration-file while parsing */
 {   struct tokinfo_t {
         const char *name;
         int varoffset;
@@ -337,7 +494,7 @@ static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
 
 
 tgtdefn_t *parse_stream(FILE *fp, const char *cfgname)
-    /* Convert config-file information into linked-list of target-structures */
+    /** Convert config-file information into linked-list of target-structures */
 {   enum { C_SPACE, C_WORD, C_COMMENT };
     int ch, lineno=1, literal=0;
     unsigned pos=0, bufflen=0;
@@ -422,7 +579,7 @@ tgtdefn_t *parse_stream(FILE *fp, const char *cfgname)
 
 
 tgtdefn_t *parse_config(const char *cfgname)
-    /* Convert config-file into linked-list of target-structures */
+    /** Convert config-file into linked-list of target-structures */
 {   FILE *fp;
     tgtdefn_t *head=NULL;
 
@@ -440,7 +597,7 @@ tgtdefn_t *parse_config(const char *cfgname)
 
 
 tgtdefn_t *parse_config_fd(int fd)
-    /* Convert input-stream config-data into target-structures */
+    /** Convert input-stream config-data into target-structures */
 {   FILE *fp=NULL;
     tgtdefn_t *head=NULL;
     char label[64];
@@ -459,7 +616,7 @@ tgtdefn_t *parse_config_fd(int fd)
 
 
 void free_config(tgtdefn_t **head)
-    /* Free all entries in target-config list */
+    /** Free all entries in target-config list */
 {   tgtdefn_t *cmx;
 
     if (head == NULL) return;
@@ -473,7 +630,7 @@ void free_config(tgtdefn_t **head)
 
 
 tgtstat_t *alloc_tgtstatus(const tgtdefn_t *tgt)
-    /* Create new status record for given target */
+    /** Create new status record for given target */
 {   tgtstat_t *ts;
 
     ts = (tgtstat_t*)malloc(sizeof(tgtstat_t));
@@ -488,7 +645,7 @@ tgtstat_t *alloc_tgtstatus(const tgtdefn_t *tgt)
 
 
 void free_tgtstatus(tgtstat_t *ts)
-    /* Free storage of target-status record (or list thereof) */
+    /** Free storage of target-status record (or list thereof) */
 {   tgtstat_t *tx=NULL;
 
     while ((tx = ts) != NULL) {
@@ -500,7 +657,7 @@ void free_tgtstatus(tgtstat_t *ts)
 
 
 struct statfile *statfile_open(const char *fname, const char *mode)
-    /* Prepare status-file for reading/writing */
+    /** Prepare status-file for reading/writing */
 {   struct statfile *sf=NULL;
     char buff[256];
     FILE *fp;
@@ -527,7 +684,7 @@ struct statfile *statfile_open(const char *fname, const char *mode)
 
 
 tgtstat_t *statfile_read(struct statfile *sf)
-    /* Read information about next target from status-file */
+    /** Read information about next target from status-file */
 {   tgtstat_t *ts=NULL;
     char *ident=NULL;
     int len;
@@ -575,7 +732,7 @@ void statfile_close(struct statfile *sf)
 
 
 tgtstat_t *get_tgtstatus(const tgtdefn_t *tgt)
-    /* Find mount/owner status of given target */
+    /** Find mount/owner status of given target */
 {   char *fname=NULL;
     tgtstat_t *ts=NULL;
     struct statfile *sf;
@@ -603,7 +760,7 @@ tgtstat_t *get_tgtstatus(const tgtdefn_t *tgt)
 
 
 tgtstat_t *get_all_tgtstatus()
-    /* Find list of mount/owner status for all mounted targets */
+    /** Find list of mount/owner status for all mounted targets */
 {   char *fname=NULL;
     tgtstat_t *ts=NULL, *head=NULL, **sfp=&head;
     struct statfile *sf;
@@ -687,5 +844,5 @@ int put_tgtstatus(const tgtdefn_t *tgt, const tgtstat_t *newstat)
 }
 
 /*
- *  (C)Copyright 2005-2011, RW Penney
+ *  (C)Copyright 2005-2012, RW Penney
  */
